# File hfibarea.cl
# By Doug Mink, after a shell script by Brian McLeod
# December 14, 2006

# HFIBAREA -- Compute and return the area of sky subtended by a specific
#             Hectospec or Hectochelle fiber

procedure fibarea

char	spra="0:00"	{prompt="Right ascension of plate center"}
char	spdec="0:00"	{prompt="Declination of plate center"}
char	sra="0:00"	{prompt="Right ascension of fiber"}
char	sdec="0:00"	{prompt="Declination of fiber"}
char	sx="0.0"	{prompt="X coordinate of fiber on focal plane"}
char	sy="0.0"	{prompt="Y coordinate of fiber on focal plane"}

char	sarea=0.0	{prompt="Area of fiber on the sky (returned)"}
bool	verbose=no	{prompt="Print table of areas if yes"}
bool	debug=no	{prompt="Print table of positions if yes"}

begin

	real	pra, pdec, ra, dec, angle, area
	real	r, radial, tangential
	real	deg, deg2, deg3, deg4, deg5
	real	xr1, xr2, yr1, yr2, pi, deg2rad, rad2deg
	real	pos10, pos11, pos12, pos20, pos21, pos22, w, diff, cosb
	bool	vb, db

	vb = verbose
	db = debug

# Direction of plate center (Convert RA to degrees if sexigesimal)
	pra = real (spra)
	if (stridx (":", spra) > 0) {
	    pra = pra * 15.d0
	    }
	pdec = real (spdec)

# Direction for this fiber (Convert RA to degrees if sexigesimal)
	ra = real (sra)
	if (stridx (":", sra) > 0) {
	    ra = ra * 15.d0
	    }
	dec = real (sdec)

	if (db) {
            printf ("pra=%9.6f pdec=%9.6f ra=%9.6f dec=%9.6f\n", pra, pdec, ra, dec)
	    }

# Compute angle from optical axis (pointing direction) to fiber
	pi =  3.141592653589793238462643d0
	deg2rad = pi / 180.d0
	rad2deg = 180.d0 / pi

#  Convert two vectors to direction cosines
	xr1 = pra * deg2rad
	yr1 = pdec * deg2rad
	cosb = cos (yr1)
	pos10 = cos (xr1) * cosb
	pos11 = sin (xr1) * cosb
	pos12 = sin (yr1)

	xr2 = ra * deg2rad
	yr2 = dec * deg2rad
	cosb = cos (yr2)
	pos20 = cos (xr2) * cosb
	pos21 = sin (xr2) * cosb
	pos22 = sin (yr2)

# Modulus squared of half the difference vector
	w = 0.0
	w = w + (pos10 - pos20) * (pos10 - pos20)
	w = w + (pos11 - pos21) * (pos11 - pos21)
	w = w + (pos12 - pos22) * (pos12 - pos22)
	w = w / 4.0
	if (w > 1.0) w = 1.0

# Angle beween the vectors
	diff = 2.0 * atan2 (sqrt (w), sqrt (1.0 - w))
	angle = diff * rad2deg

# Compute area of fiber in square arcseconds
	deg = angle
	deg2 = deg * deg
	deg3 = deg2 * deg
	deg4 = deg3 * deg
	deg5 = deg4 * deg

	r = 599.9472d0 * deg + 37.369d0 * deg3 + 28.0281d0 * deg5

	radial = (599.9472d0 + 3.d0*37.369d0*deg2 + 5.d0*28.0281d0*deg4) / 3600.d0;

	if ( deg == 0 )
            tangential = radial
	else
            tangential = r / deg / 3600.d0

	area = 3.1415926535d0 / radial / tangential * 0.125d0 * 0.125d0;

	if (db) {
            printf ("ang=%7.3f r=%7.3f rad=%8.6f tan=%8.6f area=%8.6f\n", deg, r, radial, tangential, area)
	    }
	if (vb) {
            printf ("pra=%9.6f pdec=%9.6f ra=%9.6f dec=%9.6f area=%8.6f\n", pra, pdec, ra, dec, area)
	    }

	sarea = area

	return
end
