# HFLAT1 -- Divide hectospec image by pixel normalization file from apflatten
# By Doug Mink, SAO Telescope Data Center
# February 22, 2008

procedure hflat1 (images)

string	images = ""	{prompt="Input multispec image(s) to correct"}
string	normfile=""	{prompt="Pixel to pixel normalization file from apflatten"}
bool saveold = yes	{prompt="Save original file to Nonorm/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb
	file imin, imout, imnorm, newfile
	int lroot
	string history

	imin = image
	vb = verbose
	imnorm = normfile


# Do not flatten spectra twice
	imgetp.default = "0"
	imgetp (imin, "HFLAT")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HFLAT1: ", imin, " has been processed")
		print ("HFLAT = ", imgetp.value)
		}
	    return
	    }
	else {
	    imgetp (imin, "PIXNORM")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HFLAT: ", imin, " has been processed")
		    }
		return
		}
	    }

# Remove extension from input file, if one is present, and make output file name
	smatch (imin, ".fits")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".flattened.fits"
	    }
	else {
	    imout = imin // ".flattened.fits"
	    imin = imin // ".fits"
	    }

# Add FITS extension to normalization file if it is missing
	smatch (imnorm, ".fits")
	if (!smatch.found) {
	    imnorm = imnorm // ".fits"
	    }
	if (vb) {
	    print ("HFLAT: ", imin," -> ",imout," using ", imnorm)
	    }

# Divide out normalization spectra
	imarith (imin, "/", imnorm, imout)

# Move original file to Unflat directory
	if (saveold) {
	    if (!access ("Unflat/")) {
		mkdir ("Unflat")
		}
	    newfile = "Unflat/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Unflat")
	    if (vb) {
		print ("HFLAT: Original ", imin, " => Unflat/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imout, imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " pixels normalized by " // imnorm
        hedit (imin, "HFLAT", history, add+, verify-)

end
# Feb 22 2008	New program based on HFLAT
