# File hectospec/Scripts/hgain1.cl
# By Doug Mink after Bill Wyatt
# June 2, 2008

# Run the 4 separate gain corrections for HECTOSPEC

procedure hgain1 (image)

string image = ""        {prompt="Image to correct"}

string	gain1 = "!GAIN"	{prompt="Gain for amplifier 1"}
string	gain2 = "!GAIN"	{prompt="Gain for amplifier 2"}
string	gain3 = "!GAIN"	{prompt="Gain for amplifier 3"}
string	gain4 = "!GAIN"	{prompt="Gain for amplifier 4"}
bool	saveold=yes     {prompt="Save unprocessed image files (y or n)"}
bool	deltemp=yes	{prompt="Save single-extension files (y or n)"}
bool	verbose=yes	{prompt="Verbose output (y or n)"}
bool	debug=no	{prompt="Extra debugging output (y or n)"}

begin

	string imroot, gcroot, imx, gainkey
	string gcsect0, gcsect1, gcsect2, gcsect3, gcsect4
	string imsect0, imsect1, imsect2, imsect3, imsect4, val0, val
	file imin, imu
	bool vb, db, dt, sv
	real g1, g2, g3, g4

	vb = verbose
	db = debug
	dt = deltemp
	sv = saveold
	imin = image
	imx = ".fits"

# Do not process images without extensions
	imgetp.default = "-1"
	imsect0 = imin // "[0]"
	imgetp (imsect0, "i_naxis")
	if (imgetp.value != "0") {
	    print ("HGAIN1: ", imin, " is not a multi-extension image")
	    print ("NAXIS[0] = ", imgetp.value)
	    return
	    }

#  If amplifier gain has already been done, return
	imgetp.default = "0" 
	imsect1 = imin // "[1]"
	imsect2 = imin // "[2]"
	imsect3 = imin // "[3]"
	imsect4 = imin // "[4]"
	imgetp (imsect1, "GAINCORR")
	if (imgetp.value != "0") {
	    print ("HPROC: GAINCORR has already been run on", imin)
	    print ("GAINCORR: ", imgetp.value)
	    imgetp (imsect2, "GAINCORR")
	    print ("GAINCORR: ", imgetp.value)
	    imgetp (imsect3, "GAINCORR")
	    print ("GAINCORR: ", imgetp.value)
	    imgetp (imsect4, "GAINCORR")
	    print ("GAINCORR: ", imgetp.value)
	    return
	    }

	if(vb)
	    printf("GAINCORR: Divide gain from extensions of %s\n",imin)

# Get gain for each extension of image
	if (gain1 == "") {
	    g1 = 0.0
	    }
	else if (substr (gain1, 1, 1) == "!") {
	    gainkey = substr (gain1, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect1, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 1 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		g1 = 0.0
		}
	    else
		g1 = real (imgetp.value)
	    }
	else {
	    g1 = real (gain1)
	    }

	if (gain2 == "") {
	    g2 = 0.0
	    }
	else if (substr (gain2, 1, 1) == "!") {
	    gainkey = substr (gain2, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect2, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 2 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		g2 = 0.0
		}
	    else
		g2 = real (imgetp.value)
	    }
	else {
	    g2 = real (gain2)
	    }

	if (gain3 == "") {
	    g3 = 0.0
	    }
	else if (substr (gain3, 1, 1) == "!") {
	    gainkey = substr (gain3, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect3, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 3 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		g3 = 0.0
		}
	    else
		g3 = real (imgetp.value)
	    }
	else {
	    g3 = real (gain3)
	    }

	if (gain4 == "") {
	    g4 = 0.0
	    }
	else if (substr (gain4, 1, 1) == "!") {
	    gainkey = substr (gain4, 2, 9)
	    imgetp.default = "0" 
	    imgetp (imsect4, gainkey)
	    if (imgetp.value == "0") {
		print ("HGAIN1: Gain 4 keyword ", gainkey, " not found")
		print ("HGAIN1: No gain correction can be done")
		g4 = 0.0
		}
	    else
		g4 = real (imgetp.value)
	    }
	else {
	    g4 = real (gain4)
	    }

	if (g1 == 0.0) {
	    if (g2 == 0.0) {
		if (g3 == 0.0) {
		    if (g4 == 0.0) {
			print ("HGAIN1: Gains 1,2,3,4 are zero; no correction")
			return
			}
		    else {
			print ("HGAIN1: Gains 1,2,3 are zero; no correction")
			return
			}
		    }
		else if (g4 == 0) {
		    print ("HGAIN1: Gains 1,2,4 are zero; no correction")
		    return
		    }
		else {
		    print ("HGAIN1: Gains 1and 2 are zero; no correction")
		    return
		    }
		}
	    else if (g3 == 0.0) {
		if (g4 == 0.0) {
		    print ("HGAIN1: Gains 1,3,4 are zero; no correction")
		    return
		    }
		else {
		    print ("HGAIN1: Gains 1 and 3 are zero; no correction")
		    return
		    }
		}
	    else if (g4 == 0.0) {
		print ("HGAIN1: Gains 1 and 4 are zero; no correction")
		return
		}
	    }
	else if (g2 == 0.0) {
	    if (g3 == 0.0) {
		if (g4 == 0.0) {
		    print ("HGAIN1: Gains 2,3,4 are zero; no correction")
		    return
		    }
		else {
		    print ("HGAIN1: Gains 2 and 3 are zero; no correction")
		    return
		    }
		}
	    else if (g4 == 0.0) {
		print ("HGAIN1: Gains 2 and 4 are zero; no correction")
		return
		}
	    else {
		print ("HGAIN1: Gain 2 is zero; no correction")
		return
		}
	    }
	else if (g3 == 0.0) {
	    if (g4 == 0.0) {
		print ("HGAIN1: Gains 3 and 4 are zero; no correction")
		return
		}
	    else {
		print ("HGAIN1: Gain 3 is zero; no correction")
		return
		}
	    }
	else if (g4 == 0.0) {
	    print ("HGAIN1: Gain 4 is zero; no correction")
	    return
	    }

# Find root name of image
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imin
	    imin = imin // imx
	    }
   
# Set output file name root
	gcroot = imroot // "gc"

# Assign file names for extensions of output files
	gcsect0 = gcroot // "_0.fits"
	if (access (gcsect0))
	    delete (gcsect0)
	gcsect1 = gcroot // "_1.fits"
	if (access (gcsect1))
	    delete (gcsect1)
	gcsect2 = gcroot // "_2.fits"
	if (access (gcsect2))
	    delete (gcsect2)
	gcsect3 = gcroot // "_3.fits"
	if (access (gcsect3))
	    delete (gcsect3)
	gcsect4 = gcroot // "_4.fits"
	if (access (gcsect4))
	    delete (gcsect4)

# Multiply each amplifier image by appropriate gain correction
	imcopy (imsect0, gcsect0, verbose=vb)
	imarith (imsect1, '/', g1, gcsect1, verbose=db)
	imarith (imsect2, '/', g2, gcsect2, verbose=db)
	imarith (imsect3, '/', g3, gcsect3, verbose=db)
	imarith (imsect4, '/', g4, gcsect4, verbose=db)

# Add GAINCORR keyword to and remove GAIN keyword from header
	hedit.show = no
	gettime ()
	val0 = gettime.date // " " // gettime.time
	val = val0 // " Corrected amp 1 for gain of " // g1
	hedit (gcsect1, "GAINCORR", val, add=yes, verify=no)
	hedit (gcsect1, "GAIN", "1.0", add=yes,update=yes,verify=no)
	if (vb)
	    print ("HGAIN1: ",val)
	val = val0 // " Corrected amp 2 for gain of " // g2
	hedit (gcsect2, "GAINCORR", val, add=yes, verify=no)
	hedit (gcsect2, "GAIN", "1.0", add=yes,update=yes,verify=no)
	if (vb)
	    print ("HGAIN1: ",val)
	val = val0 // " Corrected amp 3 for gain of " // g3
	hedit (gcsect3, "GAINCORR", val, add=yes, verify=no)
	hedit (gcsect3, "GAIN", "1.0", add=yes,update=yes,verify=no)
	if (vb)
	    print ("HGAIN1: ",val)
	val = val0 // " Corrected amp 4 for gain of " // g4
	hedit (gcsect4, "GAINCORR", val, add=yes, verify=no)
	hedit (gcsect4, "GAIN", "1.0", add=yes,update=yes,verify=no)
	if (vb)
	    print ("HGAIN1: ",val)

#  Save original file in Ungain/ directory, if requested
	if (sv) {
	    if (!access ("Ungain/")) {
		mkdir ("Ungain")
		}
	    imu = "Ungain/" // imin
	    if (access (imu)) {
		delete (imu)
		}
	    rename (imin, imu)
	    if (vb)
		print ("HGAIN1: Original file moved to ",imu)
	    }

# Otherwise, delete the original file
	else {
	    delete (imin)
	    if (vb)
		print ("HGAIN1: Original file ",imin," deleted")
	    }

#  Rejoin result to same name as original file
	mscjoin (gcroot, output=imin, delete=dt, verbose=vb)
	if (vb)
	    print("HGAIN1: rejoined ", imin)

end

# Feb 22 2008	New task to gain correct one image, based on GAINCORRECT
# Feb 29 2008	Updated gains for Hectospec from Nelson Caldwell
# Mar  3 2008	Divide by gain rather than multiplying by correction factor
# Mar  3 2008	Optionally read gain from GAIN header keywords
# Mar  3 2008	Change GAINCORR keyword to be more explicit
# Mar  5 2008	Delete GAIN keywords from output file
# Jun  2 2008	Make Ungain directory if it does not already exist
