# HGEOM1 -- Correct multispec spectrum image for fiber area coverage
# By Doug Mink, SAO/TDC
# May 27, 2005

procedure hgeom1 (image)

file	image = ""	{prompt="Input multispec image to correct"}
bool	deltemp = no	{prompt="Delete temporary files (y or n)"}
bool	saveold = yes	{prompt="Save unprocessed files to Nogeo/ (y or n)"}
bool	verbose = yes	{prompt="Log progress (y or n)"}
bool	debug = no	{prompt="Log each aperture (y or n)"}

begin
	bool vb, db
	string	imtype, mstype, keyroot, apxkey, apykey, rratio, aps
	string apkey
	file imin, imout, geores, geotemp, imroot, newfile
	string imageap, history, order, function, ims, tchar, apid
	int index, iap, nap, npix, lroot, xcol, ycol
	real x, y, rad, ratio

	imin = image
	vb = verbose
	db = debug

#  Create input multispec file name from input name
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imout = substr (imin, 1, lroot) // ".corr.ms.fits"
	    imin = substr (imin, 1, lroot) // ".ms.fits"
	    }
	else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imin = substr (imin, 1, lroot) // ".ms.fits"
		}
	    else {
		imin = imin // ".ms.fits"
		}
	    if (!access (imin)) {
		print ("HGEOM1: ", imin," does not exist")
		return
		}
	    }

# Check to see whether geometrical flux correction has been done
	imgetp.verbose = db
	imgetp.default = "0"
	imgetp (imin, "HGEOM")
	if (imgetp.value == "0") {
	    imgetp (imin, "FIBRGEOM")
	    }
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HGEOM1: ", imin, " has already been geometrically corrected")
		print (" HGEOM = ", imgetp.value)
		}
	    return
	    }
	else if (vb) {
	    print ("HGEOM1: ", imin," -> ",imout)
	    }

# Get number of apertures and pixels from header of input multispec image
	imgetp.verbose = no
	imgetp.default = "0"
	imgetp (imin, "i_naxis1")
	npix = int (imgetp.value)
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)
	if (vb) {
	    print ("HGEOM1: ", nap," apertures of ",npix," pixels each")
	    }

# Set up temporary geometric correction file
	geotemp = mktemp ("geocorr") // ".ms.fits"
	if (vb) {
	    print ("HGEOM1: radius normalization file is ", geotemp)
	    }
	chpixtype (imin, geotemp, "real")

# Compute geometric correction for each fiber based on its position
	for (iap = 1; iap <= nap; iap = iap + 1) {
	    apxkey = "APID" // iap // ",6"
	    imgetp.default = ""
	    imgetp (imin, apxkey)
	    if (imgetp.value == "") {
		xcol = 3
		ycol = 4
		}
	    else {
		xcol = 6
		ycol = 7
		}

# If second token is _, object name has been split in two
	    apkey = "APID" // iap // ",2"
	    imgetp.default = 9999.0
	    imgetp (imin, apkey)
	    if (imgetp.value == "_") {
		xcol = 7
		ycol = 8
		}

# If third token is _, object name has been split in three
	    else {
		apkey = "APID" // iap // ",3"
		imgetp.default = 9999.0
		imgetp (imin, apkey)
		if (imgetp.value == "_") {
		    xcol = 8
		    ycol = 9
		    }
		}

# Extract x and y focal plane coordinates
	    apxkey = "APID" // iap // "," //xcol
	    imgetp.default = 9999.0
	    imgetp (imin, apxkey)
	    x = real (imgetp.value)
	    apykey = "APID" // iap // "," //ycol
	    imgetp (imin, apykey)
	    y = real (imgetp.value)

# Compute pixel size correction for this aperture
	    rad = sqrt (x * x + y * y)
	    ratio = 1.11748 + (0.0001866 * rad) - (0.00000188622 * rad * rad);
	    rratio = ratio
	    imageap = geotemp // "[*," // iap // "]"
	    if (db) {
		apkey = "APID" // iap
		imgetp (imin, apkey)
		print (apkey,":", imgetp.value)
		print ("HGEOM1: ", imin,iap, x, y, rad, ratio)
		}
	    imreplace (imageap, ratio)
	    }

# Correct all spectra for optical projection differences
	geores = mktemp ("geores") // ".ms.fits"
	imarith (imin, "/", geotemp, geores, verbose=yes)
	if (deltemp) {
	    delete (geotemp)
	    }

# Add history line including time about geometry correction
	gettime ()
	history = gettime.date // " " // gettime.time // " geometric correction made"
	hedit (geores, "HGEOM", history, add+, verify-)

# Move original file to Nogeo directory
	if (saveold) {
	    if (!access ("Nogeo/")) {
		mkdir ("Nogeo")
		}
	    newfile = "Nogeo/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Nogeo")
	    }
	else {
	    delete (imin)
	    }
	imrename (geores, imin)
end

# Mar  9 2005	New task split off from HGEOM
# May 27 2005	Deal with different numbers of tokens when object name is weird
