# HPIXSHIFT - Use correlation or emission line to find multispec spectra shift
# June 30, 2005
# By Doug Mink, SAO Telescope Data Center

procedure hpixshift (images)

string	images=""	{prompt="Image file(s) for which to compute shifts"}

int	shiftline=1	{prompt="Line to shift in position file"}
file	emlines="hectosky1.dat"	{prompt="File of emission line positions"}
file	linedir="rvsao$lib/"	{prompt="Directory for emission line positions"}
string	aprange="50-100" {prompt="Apertures over which to average pixel shift"}
file	refimage=""	{prompt="Night sky template image (time near flat)"}
bool	first=yes	{prompt="Template earliest (yes) or latest (no) image"}
bool	saveshift=yes	{prompt="Save shift in header (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print shifts (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin

file	templist, tempdirs, tempfiles, tempemsao
file	imin, imref
string	ims, history, tchar, nline, result
string	filetemp, arg, temp, dateobs, cshift
real	shift, wl1, mjd, mjd0
bool	vb, db, firstfile, last, correlate

	if (first) {
	    last = no
	    }
	else {
	    last = yes
	    }

	vb = verbose
	db = debug
	ims = images
	imref = refimage

	if (ims == "") {
	    print ("HPIXSHIFT: Find pixel shift in multispec night sky spectra")
	    print ("           Run this task on files which have been dispersion-corrected,")
	    print ("           but not linearized, where pixels are not linear in wavelength.")
	    print ("Usage: hpixshift <input images>")
	    print ("Usage: hpixshift @<input image list file>")
	    return
	    }

#  Figure out which line to check
	if (shiftline == 0) {
	    correlate = yes
	    }
	else {
	    nline = shiftline + 8
	    correlate = no
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("shiftlist")
	    files (ims, >templist)
	    }
	if (vb) {
	    print ("HPIXSHIFT: Reading from list file ", templist)
	    if (correlate) {
		print ("HPIXSHIFT: Reference image is ", imref)
		}
	    else {
		print ("HPIXSHIFT: Shift measured from ", shiftline)
		}
	    }

#  If correlating, use earliest file in list as reference if none is set
	if (correlate && imref == "") {
	    mjd0 = 0.0
            list = templist
            while (fscan (list,imin) != EOF) {
		imgetp.default = "0"
		imgetp (imin, "MJD")
		if (imgetp.value != "0") {
		    mjd = real (imgetp.value)
		    }
		if (mjd0 == 0.0) {
		    mjd0 = mjd
		    imref = imin
		    }
		else if (first && mjd < mjd0) {
		    mjd0 = mjd
		    imref = imin
		    }
		else if (last && mjd > mjd0) {
		    mjd0 = mjd
		    imref = imin
		    }
		}
	    if (vb) {
		imgetp (imref, "DATE-OBS")
		dateobs = imgetp.value
		print ("HPIXSHIFT: ", imref, " at ",dateobs," is reference")
		}
	    }

#  Set up xcsao parameters
	if (correlate) {
	    xcsao.specnum = aprange
	    xcsao.specband = 0
	    xcsao.specdir = ""
	    xcsao.correlate = "pixel"
	    xcsao.templates = 
	    xcsao.tempnum = ""
	    xcsao.tempband = 0
	    xcsao.tempdir = ""
	    xcsao.echelle = yes
	    xcsao.st_lambda = INDEF
	    xcsao.end_lambda = 8500.
	    xcsao.limfile = ""
	    xcsao.obj_plot = no
	    xcsao.xcor_plot = no
	    xcsao.xcor_file = no
	    xcsao.fixbad = no
	    xcsao.badlines = "badlines.dat"
	    xcsao.s_emchop = "no"
	    xcsao.t_emchop = "no"
	    xcsao.s_abs_reject = 0.
	    xcsao.s_em_reject = 0.
	    xcsao.t_abs_reject = 0.
	    xcsao.t_em_reject = 0.
	    xcsao.bell_window = 0.05
	    xcsao.renormalize = no
	    xcsao.ncols = 8192
	    xcsao.interp_mode = "spline3"
	    xcsao.zeropad = "yes"
	    xcsao.low_bin = 5
	    xcsao.top_low = 20
	    xcsao.top_nrun = 8192
	    xcsao.nrun = 8292
	    xcsao.vel_init = "zero"
	    xcsao.czguess = 0.
	    xcsao.nzpass = 0
	    xcsao.tshift = 0.
	    xcsao.svel_corr = "file"
	    xcsao.tvel_corr = "none"
	    xcsao.pkmode = 1
	    xcsao.pkfrac = 0.5
	    xcsao.pksrch = 25
	    xcsao.minvel = -5000.
	    xcsao.maxvel = 150000.
	    xcsao.report_mode = -12
	    xcsao.logfiles = "xcsao.log,STDOUT"
	    xcsao.save_vel = no
	    xcsao.rvcheck = no
	    xcsao.archive = no
	    xcsao.nsmooth = 5
	    xcsao.cvel = INDEF
	    xcsao.dvel = INDEF
	    xcsao.ablines = ""
	    xcsao.emlines = emlines
	    xcsao.linedir = linedir
	    xcsao.dispmode = 1
	    xcsao.displot = yes
	    xcsao.device = "stdgraph"
	    xcsao.vel_plot = "correlation"
	    xcsao.curmode = no
	    xcsao.hardcopy = no
	    xcsao.plotter = "stdplot"
	    xcsao.temp_plot = no
	    xcsao.contsub_plot = no
	    xcsao.apodize_plot = no
	    xcsao.fft_plot = no
	    xcsao.tfft_plot = no
	    xcsao.uxcor_plot = no
	    xcsao.nsum = 1
	    xcsao.debug = no
	    xcsao.cursor = ""
	    xcsao.mode = "ql"
	    }

#  Set up emsao parameters
	else {
	    emsao.specnum = aprange
	    emsao.specband = 1
	    emsao.skynum = ""
	    emsao.skyband = 0
	    emsao.specdir = ""
	    emsao.linefit = yes
	    emsao.fixbad = no
	    emsao.badlines = "badlines.dat"
	    emsao.renormalize = no
	    emsao.st_lambda = INDEF
	    emsao.end_lambda = INDEF
	    emsao.nsmooth = 0
	    emsao.vel_init = "guess"
	    emsao.czguess = 0.
	    emsao.cortemp = ""
	    emsao.wspan = 10.
	    emsao.linesig = 1.
	    emsao.emsearch = ""
	    emsao.emlines = emlines
	    emsao.linedir = linedir
	    emsao.npfit = 2
	    emsao.nlcont = 1
	    emsao.esmooth = 2
	    emsao.emcombine = ""
	    emsao.mincont = 0.
	    emsao.lwmin = 0.4
	    emsao.lwmax = 1.7
	    emsao.lsmin = 1.
	    emsao.sigline = 0.
	    emsao.disperr = 0.01
	    emsao.vel_corr = "none"
	    emsao.report_mode = 11
	    emsao.archive = no
	    emsao.save_vel = no
	    emsao.verbose = vb
	    emsao.device = "stdgraph"
	    emsao.hardcopy = no
	    emsao.displot = db
	    emsao.plotter = "stdplot"
	    emsao.dispmode = 2
	    emsao.vel_plot = "emission"
	    emsao.curmode = no
	    emsao.dispem = yes
	    emsao.dispabs = no
	    emsao.ablines = "ablines.dat"
	    emsao.obj_plot = no
	    emsao.contsub_plot = no
	    emsao.debug = no
	    emsao.nsum = 1
	    emsao.cursor = ""
	    }

# Loop through files
        list = templist
        while (fscan (list, imin) != EOF) {

	    imgetp.default = "0"
	    imgetp (imin, "HPIXSHIFT")
	    if (imgetp.value != "0") {
		print ("HPIXSHIFT has already been run.")
		print ("HPIXSHIFT: ", imgetp.value)
		next
		}

#	Compute shift of sky spectrum using XCSAO
	    if (correlate) {
		}

#	Compute shift of selected line using EMSAO
	    else {
        	tempemsao = mktemp ("shiftemsao")
		if (db) {
		    emsao.logfiles = "STDOUT," // tempemsao
		    }
		else {
		    emsao.logfiles = tempemsao
		    }
		if (debug) {
		    dpar emsao
		    }
#		print ("HPIXSHIFT: emsao -> ",tempemsao)
		emsao (imin)

# Compute mean pixel shift for selected emission line and read it back
		arg = "-mp " // tempemsao // " " // nline
		getcol (arg) | scan (shift)

# Delete temporary file for this image
		if (deltemp) {
		    delete (tempemsao)
		    }
		}

# Add pixel shift to header
	    gettime ()
	    history = gettime.date // " " // gettime.time // " " // shift // " pixel shift from " // aprange
	    if (vb) {
		print ("HPIXSHIFT: ", imin, ": ", history)
		}
	    hedit (imin, "HPIXSHIF", history, add=yes, verify=no, show=db)
	    cshift = shift
	    hedit (imin, "PIXSHIFT", cshift, add=yes, verify=no, show=db)
	    }

# Delete temporary file for this pointing
	if (deltemp && tchar != '@') {
	    delete (tempfiles)
	    }
end

# May 20 2005	Task to compute night sky spectrum shifts
# May 26 2005	Identify line from which to compute shift by its number
# May 27 2005	Add emlines = file of line positions, linedir = directory
# Jun  1 2005	Add apertures to HPIXSHIF keyword value
# Jun 30 2005	Load rvsao and wcstools in hectospec.cl
