# File hectospec/Scripts/hskysub1.cl
# By Doug Mink
# September 7, 2007

procedure hskysub1 (objskyfile, skyfile, outfile)

file	objskyfile=""	{prompt="Object plus sky file name"}
file	skyfile=""	{prompt="Sky file name"}
file	outfile=""	{prompt="Output object-sky file name"}

int     objap=1         {prompt="Aperture for object+sky spectrum"}
int     objband=1       {prompt="Band for object+sky spectrum"}
file    objdir=""       {prompt="Directory containing object+sky spectrum"}
int     skyap=1         {prompt="Aperture for sky spectrum"}
int     skyband=1       {prompt="Band for sky spectrum"}
file    skydir=""       {prompt="Directory containing sky spectrum"}
file    outdir=""       {prompt="Directory for output spectrum"}
file	skyline="OH8399.dat"	{prompt="Sky line file to use for scaling factor"}
file	linedir="hectospec$lib/"	{prompt="Directory for sky line data"}
bool	usearea=no	{prompt="Use sky line area vs. height in scaling"}
bool    plotres = no    {prompt="Plot sky-subtracted spectrum (y or n)"}
bool	deltemp=yes	{prompt="Delete temporary files (y or n)"}
bool	verbose=yes	{prompt="note progress"}
bool	debug=no	{prompt="note progress in detail"}

begin
	bool vb, dt, db, diffobj, diffsky, plotr
	real skyfac, objfac, skynorm
	file normsky, objfile, osfile, sfile, ofile, ostemp, outpath
	string files

	if (!deftask ("emsao"))
	    rvsao (motd=no)

	if (!defpac ("wcstools"))
	    wcstools (motd=no)

	dt = deltemp
	vb = verbose
	db = debug
	ofile = mktemp ("object") // ".fits"
	osfile = objskyfile
	sfile = skyfile
	plotr = plotres

# Check for existence of input files
	if (!access (osfile)) {
	    print ("HSKYSUB1: Cannot read object file ",osfile)
	    return
	    }
	if (!access (sfile)) {
	    print ("HSKYSUB1: Cannot read sky file ",sfile)
	    return
	    }
	normsky = mktemp ("skynorm") // ".fits"

# Initialize EMSAO for background normalization
	pemsao.emlines = skyline
	pemsao.linedir = linedir
	pemsao.verbose = vb
	pemsao.debug = db

# Initialize CONTPARS (continuum fit for EMSAO)
	contpars.c_interactive = no
	contpars.c_sample = "*"
	contpars.c_function = "spline3"
	contpars.naverage = 1
	contpars.order = 15
	contpars.s_low_reject = 2.
	contpars.s_high_reject = 2.
	contpars.t_low_reject = 2.
	contpars.t_high_reject = 2.
	contpars.niterate = 10
	contpars.grow = 1.

# Check to see whether same line was used before in sky
	imgetp.default = "0"
	imgetp (sfile, "EMLFILE")
	if (imgetp.value == pemsao.emlines) {
	    diffsky = no
	    if (db) {
		print ("HSKYSUB1: Using same line in sky: ",imgetp.value)
		}
	    }
	else {
	    diffsky = yes
	    if (db) {
		print ("HSKYSUB1: Using new line in sky: ",pemsao.emlines," not ",imgetp.value)
		}
	    }

# Check to see whether same line was used before in object
	imgetp.default = "0"
	imgetp (osfile, "EMLFILE")
	if (imgetp.value == pemsao.emlines) {
	    diffobj = no
	    if (db) {
		print ("HSKYSUB1: Using same line in object: ",imgetp.value)
		}
	    }
	else {
	    diffobj = yes
	    if (db) {
		print ("HSKYSUB1: Using new line in object: ",pemsao.emlines," not ",imgetp.value)
		}
	    }

# Normalize using area under selected emission line
	if (usearea) {

# Check sky header for normalization factor; compute if necessary
	    imgetp.default = "0"
	    imgetp (sfile, "EMLEQW")
	    if (imgetp.value == "0" || diffsky) {
		pemsao (sfile, specnum=skyap, specband=skyband, specdir=skydir)
		skyfac = pemsao.lineeqw
		hedit (sfile, "EMFILE", pemsao.emlines, add+, update+, show-, verify-)
		hedit (sfile, "EMLEQW", pemsao.lineeqw, add+, update+, show-, verify-)
		hedit (sfile, "EMLHGT", pemsao.lineheight, add+, update+, show-, verify-)
		}
	    else {
		skyfac = imgetp.value
		}
	    if (db)
		print ("HSKYSUB1: Scaling sky by EQW = ", skyfac)

# Check object header for normalization factor; compute if necessary
	    imgetp.default = "0"
	    imgetp (osfile, "EMLEQW")
	    if (imgetp.value == "0" || diffobj) {
		pemsao (osfile,specnum=objap, specband=objband, specdir=objdir)
		objfac = pemsao.lineeqw
		hedit (osfile, "EMFILE", pemsao.emlines, add+, update+, show-, verify-)
		hedit (osfile,"EMLEQW",pemsao.lineeqw,add+,update+,show-,verify-)
		hedit (osfile, "EMLHGT", pemsao.lineheight, add+, update+, show-, verify-)
		}
	    else {
		objfac = imgetp.value
		}
	    if (db)
		print ("HSKYSUB1: Scaling object by EQW = ", objfac)
	    }

# Normalize using height of selected emission line
	else {

# Check sky header for normalization factor; compute if necessary
	    imgetp.default = "0"
	    imgetp (sfile, "EMLHGT")
	    if (imgetp.value == "0" || diffsky) {
		pemsao (sfile, specnum=skyap, specband=skyband, specdir=skydir)
		skyfac = pemsao.lineheight
		hedit (sfile, "EMFILE", pemsao.emlines, add+, update+, show-, verify-)
		hedit (sfile, "EMLHGT", pemsao.lineheight, add+, update+, show-, verify-)
		hedit (sfile,"EMLEQW",pemsao.lineeqw,add+,update+,show-,verify-)
		}
	    else {
		skyfac = imgetp.value
		}
	    if (db)
		print ("HSKYSUB1: Scaling sky by HGT = ", skyfac)

# Check object header for normalization factor; compute if necessary
	    imgetp.default = "0"
	    imgetp (osfile, "EMLHGT")
	    if (imgetp.value == "0" || diffobj) {
		pemsao (osfile,specnum=objap, specband=objband, specdir=objdir)
		objfac = pemsao.lineheight
		hedit (osfile, "EMFILE", pemsao.emlines, add+, update+, show-, verify-)
		hedit (osfile,"EMLHGT",pemsao.lineheight,add+,update+,show-,verify-)
		hedit (osfile,"EMLEQW",pemsao.lineeqw,add+,update+,show-,verify-)
		}
	    else {
		objfac = imgetp.value
		}
	    if (db)
		print ("HSKYSUB1: Scaling object by HGT = ", objfac)
	    }

# Multiply sky by normalization file before subtracting
	skynorm = objfac / skyfac
	if (skyap != 1 || skyband != 1)
	    sf = skyfile // "[*," // skyap // "," // skyband // "]"
	else
	    sf = skyfile
	imarith (sf, "*", skynorm, normsky)

# Subtract normalized sky from object+sky file
	ostemp = mktemp ("osfile") // ".fits"
	if (objap != 1 || objband != 1)
	    osf = osfile // "[*," // objap // "," // objband // "]"
	else
	    osf = osfile
	imcopy (osf, ostemp)
	imarith (ostemp, "-", normsky, ofile)

# Stack object+sky and sky spectra on processes spectrum
	if (outdir != "")
	    outpath = outdir // outfile
	else
	    outpath = outfile
	files = "-mo " // outpath // " " // ofile //" "// ostemp //" "//normsky
	istack (files)

# Delete temporary processed spectrum files
	if (dt) {
	    imdelete (ofile)
	    imdelete (normsky)
	    imdelete (ostemp)
	    }
end
# Aug 31 2007	New task based on Bill Wyatt's skyproc ksh script
# Sep  7 2007	Add parameters to match options in hskyfit1
