# HVCOMBINE - Combine multispec files, removing cosmic rays in the process
# January 22, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hvcombine (images, output)

string	images=""	{prompt="Images to combine"}
file	output=""	{prompt="Output image"}
string	apertures = "1-300"	{prompt="Range of apertures to combine"}
string	combine = "average"	{prompt="Combine images (average|median|sum)"}
string	reject = "ccdclip"	{prompt="Reject method (none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip)"}
string	scale = "none"	{prompt="Image scaling"}
string	sample = ""	{prompt="Wavelength sample regions for statistics"}
bool	saveold = yes	{prompt="Save uncombined files in Uncombined/ (y or n)"}
bool	deltemp = no	{prompt="Delete intermediate image files (y or n)"}
bool	verbose = yes	{prompt="Print process progress (y or n)"}
bool	debug = no	{prompt="Print information for debugging (y or n)"}

begin

string	imagelist, templist, history, keyword
file	imageout, imin, newfile, tempkey, imvar, tempvar
bool	vb, db, samefile
int	nim, i
real	width, readnoise

	vb = verbose
	db = debug
	imagelist = images
	imageout = output

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

#  Make temporary list of input files and get name of first file
	templist = mktemp ("scomblist")
	files (imagelist, >templist)
	list = templist
	i = fscan (list, imin)
	imgetp (imin, "APID1", verbose=db)

# Save APNUMi keywords with slit positions lost during scombine
	arg = "-g " // imin // " @" // osfn ("hectospec$lib/apnumkeys")
	gethead (arg, >tempkey)

	nim = 1
	while (fscan (list,imin) != EOF) {
	    nim = nim + 1
	    }
	if (nim == 1 && imin == imageout) {
	    samefile = yes
	    imageout = mktemp ("scombfile") // ".ms.fits:"
	    }
	else {
	    samefile = no
	    }

	if (vb) {
	    print ("HSCOMBINE: ", nim, " images combined into ",imageout)
	    }

# Initialize scombine
	scombine.noutput = ""
	scombine.logfile = "STDOUT"
	scombine.apertures = apertures
	scombine.group = "apertures"
	scombine.combine = combine
	scombine.reject = reject
	scombine.first = yes
	scombine.w1 = INDEF
	scombine.w2 = INDEF
	scombine.dw = INDEF
	scombine.nw = INDEF
	scombine.log = no
	scombine.scale = scale
	scombine.zero = "none"
	scombine.weight = "none"
	scombine.sample = sample
	scombine.lthreshold = INDEF
	scombine.hthreshold = INDEF
	scombine.nlow = 1
	scombine.nhigh = 1
	scombine.nkeep = 1
	if (nim < 5) {
	    scombine.mclip = no
	    }
	else {
	    scombine.mclip = yes
	    }
	scombine.lsigma = 3.
	scombine.hsigma = 3.
	scombine.rdnoise = "2.7"
	scombine.gain = "1."
	scombine.snoise = "0."
	scombine.sigscale = 0.1
	scombine.pclip = -0.5
	scombine.grow = 0
	scombine.blank = 0.
	scombine.mode = "ql"

# Combine multispec images
	scombine (imagelist, imageout)

# Copy APID1 from first input image to output image
	hedit (imageout, "APID1", imgetp.value, add+, verify-)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " " // nim // " images scombined"
	hedit (imageout, "HSCOMBIN", history, add+, verify-)

# Make directory to save input files, if needed
	if (saveold) {
	    if (!access ("Notcombined/")) {
		mkdir ("Notcombined")
		}
	    }

# Retrieve APNUMi keywords which were lost during scombine
	arg = imageout // " @" // tempkey
	sethead (arg)

# Add list of files copied and HPIXSHIF and HLIN from each to output header
	i = 0
	list = templist
	while (fscan (list,imin) != EOF) {
	    i = i + 1
	    keyword = "HSCOMB" // i
	    hedit (imageout, keyword, imin, add+, verify-)

	    keyword = "HSHIFT" // i
	    imgetp (imin,"HPIXSHIF")
	    hedit (imageout, keyword, imgetp.value, add+, verify-)

	    keyword = "HLIN" // i
	    imgetp (imin,"HLIN")
	    hedit (imageout, keyword, imgetp.value, add+, verify-)

	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    if (saveold) {
		newfile = "Notcombined/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (imin, "Notcombined")
		}
	    }

# Make a variance file

# Find root of file name
	smatch (imageout, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imageout, 1, lroot)
	    }
	else {
	    smatch (imageout, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imageout, 1, lroot)
		}
	    else {
		imroot = imagout
		}
	    }
	imvar = imroot // "_v.ms.fits"

# Find number of files combined from output file header 
	imgetp.default = "0"
	imgetp (imout, "NCOMBINE")
	ncombine = int (imgetp.value)
	if (ncombine > 0) {
	    fcombine = real (imgetp.value)
	    }
	else {
	    fcombine = 1.0
	    }

# Compute read noise
	width = 8.0
	readnoise = (width * 2.7) * (width * 2.7) / fcombine

# Compute file of variances
	if (ncombine > 0) {
	    imtemp = mktemp ("scombvar")
	    imarith (imout, "/", fcombine, imtemp)
	    imarith (imtemp, "+", readnoise, imvar)
	    if (deltemp) {
		delete (imtemp)
		}
	    }
	else {
	    imarith (imout, "+", readnoise, imvar)
	    }

# Delete original and temporary files
	if (samefile) {
	    if (access (imin)) {
		imdelete (imin)
		}
	    imrename (imageout, imin)
	    }
	if (deltemp) {
	    delete (templist)
	    delete (tempkey)
	    }
end

# Jan 10 2005	New program to remove cosmic rays from spectra
# Feb 24 2005	Use Susan's defaults; add file transfer to subdirectory 
# Feb 24 2005	Add history, filename saving and APID1 transfer
# Feb 25 2005	Add scale and sample for images with different exposure levels
# Mar  7 2005	Allow single file to be input and output

# Dec 28 2006	Save APNUMi with slit positions and write into scombined file

# Feb 13 2007	Add variance file computation
# Mar 30 2007	Write HLIN and HPIXSHIF to HLINi and HSHIFTi of output header
# May 22 2007	Check for package wcstools instead of task sethead

# Jan 22 2008	Change float to real
